@@ -5,16 +5,23 @@
 #include <float.h>
 #include <string.h>
 #include "svm.h"
+typedef signed char schar;
 template <class T> inline T min(T x,T y) { return (x<y)?x:y; }
 template <class T> inline T max(T x,T y) { return (x>y)?x:y; }
+template <class T> inline void swap(T& x, T& y) { T t=x; x=y; y=t; }
+template <class S, class T> inline void clone(T*& dst, S* src, int n)
+{
+	dst = new T[n];
+	memcpy((void *)dst,(void *)src,sizeof(T)*n);
+}
 #define EPS_A 1e-12
-#define INF DBL_MAX
+#define INF HUGE_VAL
 
 //
 // Kernel Cache
 //
 // l is the number of total data items
-// size is the cache size
+// size is the cache size limit in bytes
 //
 class Cache
 {
@@ -22,92 +29,116 @@ public:
 	Cache(int l,int size);
 	~Cache();
 
-	// return 1 if returned data is valid (cache hit)
-	// return 0 if returned data needs to be filled (cache miss)
-	int get_data(const int index, double **data);
+	// request data [0,len)
+	// return some position p where [p,len) need to be filled
+	// (p >= len if nothing needs to be filled)
+	int get_data(const int index, double **data, int len);
+	void swap_index(int i, int j);	// future_option
 private:
 	int l;
 	int size;
 	struct head_t
 	{
 		head_t *prev, *next;	// a cicular list
-		int index;
 		double *data;
+		int len;		// data[0,len) is cached in this entry
 	};
 
 	head_t* head;
-	head_t* lru_head;
-	head_t** index_to_head;
-	void move_to_last(head_t *h);
+	head_t lru_head;
+	void lru_delete(head_t *h);
+	void lru_insert(head_t *h);
 };
 
-Cache::Cache(int _l,int _size):l(_l),size(_size)
+Cache::Cache(int l_,int size_):l(l_),size(size_)
 {
-	head = new head_t[size];
-	int i;
-	for(i=0;i<size;i++)
-	{
-		head[i].next = &head[i+1];
-		head[i].prev = &head[i-1];
-		head[i].index = -1;
-		head[i].data = new double[l];
-	}
-
-	head[0].prev = &head[size-1];
-	head[size-1].next = &head[0];
-	lru_head = &head[0];
-
-	index_to_head = new head_t *[l];
-	for(i=0;i<l;i++)
-		index_to_head[i] = 0;
+	head = (head_t *)calloc(l,sizeof(head_t));	// initialized to 0
+	size /= sizeof(double);
+	size -= l * sizeof(head_t) / sizeof(double);
+	lru_head.next = lru_head.prev = &lru_head;
 }
 
 Cache::~Cache()
 {
-	for(int i=0;i<size;i++)
-		delete[] head[i].data;
-	delete[] head;
-	delete[] index_to_head;
+	for(head_t *h = lru_head.next; h != &lru_head; h=h->next)
+		free(h->data);
+	free(head);
 }
 
-void Cache::move_to_last(head_t *h)
+void Cache::lru_delete(head_t *h)
 {
-	if(lru_head == h)
-		lru_head = lru_head->next;
-	else
-	{
-		// delete from current location
-		h->prev->next = h->next;
-		h->next->prev = h->prev;
+	// delete from current location
+	h->prev->next = h->next;
+	h->next->prev = h->prev;
+}
 
-		// insert to last position
-		h->next = lru_head;
-		h->prev = lru_head->prev;
-		h->prev->next = h;
-		h->next->prev = h;
+void Cache::lru_insert(head_t *h)
+{
+	// insert to last position
+	h->next = &lru_head;
+	h->prev = lru_head.prev;
+	h->prev->next = h;
+	h->next->prev = h;
+}
+
+int Cache::get_data(const int index, double **data, int len)
+{
+	head_t *h = &head[index];
+	if(h->len) lru_delete(h);
+	int more = len - h->len;
+
+	if(more > 0)
+	{
+		// free old space
+		while(size < more)
+		{
+			head_t *old = lru_head.next;
+			lru_delete(old);
+			free(old->data);
+			size += old->len;
+			old->data = 0;
+			old->len = 0;
+		}
+
+		// allocate new space
+		h->data = (double *)realloc(h->data,sizeof(double)*len);
+		size -= more;
+		swap(h->len,len);
 	}
+
+	lru_insert(h);
+	*data = h->data;
+	return len;
 }
 
-int Cache::get_data(const int index,double **data)
+void Cache::swap_index(int i, int j)
 {
-	head_t *h=index_to_head[index];
-	if(h)
+	if(i==j) return;
+
+	if(head[i].len) lru_delete(&head[i]);
+	if(head[j].len) lru_delete(&head[j]);
+	swap(head[i].data,head[j].data);
+	swap(head[i].len,head[j].len);
+	if(head[i].len) lru_insert(&head[i]);
+	if(head[j].len) lru_insert(&head[j]);
+
+	if(i>j) swap(i,j);
+	for(head_t *h = lru_head.next; h!=&lru_head; h=h->next)
 	{
-		move_to_last(h);
-		*data = h->data;
-		return 1;
-	}		
-	else
-	{
-		// get one from lru_head
-		h = lru_head;
-		lru_head = lru_head->next;
-		if(h->index!=-1)
-			index_to_head[h->index] = 0;
-		h->index = index;
-		index_to_head[index] = h;
-		*data = h->data;
-		return 0;
+		if(h->len > i)
+		{
+			if(h->len > j)
+				swap(h->data[i],h->data[j]);
+			else
+			{
+				// give up
+				lru_delete(h);
+				free(h->data);
+				size += h->len;
+				h->data = 0;
+				h->len = 0;
+			}
+		}
 	}
 }
 
@@ -125,14 +156,19 @@ public:
 
 	static double k_function(const svm_node *x, const svm_node *y,
 				 const svm_parameter& param);
-	virtual double *get_Q(int column) const = 0;
-
+	virtual double *get_Q(int column, int len) const = 0;
+	virtual void swap_index(int i, int j) const	// no so const...
+	{
+		swap(x[i],x[j]);
+		if(x_square) swap(x_square[i],x_square[j]);
+	}
 protected:
 
 	double (Kernel::*kernel_function)(int i, int j) const;
 
 private:
-	const svm_node * const * const x;
+	const svm_node **x;
+	double *x_square;
 
 	// svm_parameter
 	const int kernel_type;
@@ -140,7 +176,6 @@ private:
 	const double gamma;
 	const double coef0;
 
-	double *x_square;
 	static double dot(const svm_node *px, const svm_node *py);
 	double kernel_linear(int i, int j) const
 	{
@@ -160,11 +195,10 @@ private:
 	}
 };
 
-Kernel::Kernel(int l, const svm_node * const * _x, const svm_parameter& param)
-:x(_x), kernel_type(param.kernel_type), degree(param.degree),
+Kernel::Kernel(int l, const svm_node * const * x_, const svm_parameter& param)
+:kernel_type(param.kernel_type), degree(param.degree),
  gamma(param.gamma), coef0(param.coef0)
 {
-
 	switch(kernel_type)
 	{
 		case LINEAR:
@@ -184,6 +218,8 @@ Kernel::Kernel(int l, const svm_node * const * _x, const svm_parameter& param)
 			exit(1);
 	}
 
+	clone(x,x_,l);
+
 	if(kernel_type == RBF)
 	{
 		x_square = new double[l];
@@ -196,7 +232,8 @@ Kernel::Kernel(int l, const svm_node * const * _x, const svm_parameter& param)
 
 Kernel::~Kernel()
 {
-	delete x_square;
+	delete[] x;
+	delete[] x_square;
 }
 
 double Kernel::dot(const svm_node *px, const svm_node *py)
@@ -286,6 +323,7 @@ double Kernel::k_function(const svm_node *x, const svm_node *y,
 //
 //		0 <= alpha_i <= C
 //		y^T \alpha = \delta
+//		y_i = +1 or -1
 //
 // Given:
 //
@@ -297,22 +335,27 @@ double Kernel::k_function(const svm_node *x, const svm_node *y,
 //
 class Solver {
 public:
-	Solver( int _l, const Kernel& Q, const double *b, const double *_y,
-		double *_alpha, double _C, double _eps,
-		double& obj, double& rho);
-
-	~Solver();
-
-private:
-	const int l;
-	const double *y;
-	double * const alpha;
-	const double C;
-	const double eps;
-
-	double * G;		// gradient of objective function
+	Solver() {};
+	virtual ~Solver() {};
+	void Solve(int l, const Kernel& Q, const double *b_, const schar *y_,
+		   double *alpha_, double C, double eps,
+		   double& obj, double& rho, int shrinking);
+protected:
+	int active_size;
+	schar *y;
+	double *G;		// gradient of objective function
 	enum { LOWER_BOUND, UPPER_BOUND, FREE };
 	char *alpha_status;	// LOWER_BOUND, UPPER_BOUND, FREE
+	double *alpha;
+	const Kernel *Q;
+	double eps;
+	double C;
+	double *b;
+	int *active_set;
+	double *G_bar;		// gradient, if we treat free variables as 0
+	int l;
+	bool unshrinked;		// XXX
+
 	void update_alpha_status(int i)
 	{
 		if(alpha[i] >= C-EPS_A)
@@ -323,14 +366,59 @@ private:
 	}
 	bool is_upper_bound(int i) { return alpha_status[i] == UPPER_BOUND; }
 	bool is_lower_bound(int i) { return alpha_status[i] == LOWER_BOUND; }
-	int select_working_set(int &i, int &j);
+	bool is_free(int i) { return alpha_status[i] == FREE; }
+	void swap_index(int i, int j);
+	void reconstruct_gradient();
+	virtual int select_working_set(int &i, int &j);
+	virtual double calculate_rho();
+	virtual void do_shrinking();
 };
 
-Solver::Solver( int _l, const Kernel& Q, const double *b, const double *_y,
-		double *_alpha, double _C, double _eps,
-		double& obj, double& rho)
-:l(_l),y(_y),alpha(_alpha),C(_C),eps(_eps)
+void Solver::swap_index(int i, int j)
 {
+	Q->swap_index(i,j);
+	swap(y[i],y[j]);
+	swap(G[i],G[j]);
+	swap(alpha_status[i],alpha_status[j]);
+	swap(alpha[i],alpha[j]);
+	swap(b[i],b[j]);
+	swap(active_set[i],active_set[j]);
+	swap(G_bar[i],G_bar[j]);
+}
+
+void Solver::reconstruct_gradient()
+{
+	// reconstruct inactive elements of G from G_bar and free variables
+
+	if(active_size == l) return;
+
+	int i;
+	for(i=active_size;i<l;i++)
+		G[i] = C * G_bar[i] + b[i];
+	
+	for(i=0;i<active_size;i++)
+		if(is_free(i))
+		{
+			const double *Q_i = Q->get_Q(i,l);
+			double alpha_i = alpha[i];
+			for(int j=active_size;j<l;j++)
+				G[j] += alpha_i * Q_i[j];
+		}
+}
+
+void Solver::Solve(int l, const Kernel& Q, const double *b_, const schar *y_,
+		   double *alpha_, double C, double eps,
+		   double& obj, double& rho, int shrinking)
+{
+	this->l = l;
+	this->Q = &Q;
+	clone(b, b_,l);
+	clone(y, y_,l);
+	clone(alpha,alpha_,l);
+	this->C = C;
+	this->eps = eps;
+	unshrinked = false;
+
 	// initialize alpha_status
 	{
 		alpha_status = new char[l];
@@ -338,141 +426,176 @@ Solver::Solver( int _l, const Kernel& Q, const double *b, const double *_y,
 			update_alpha_status(i);
 	}
 
+	// initialize active set (for shrinking)
+	{
+		active_set = new int[l];
+		for(int i=0;i<l;i++)
+			active_set[i] = i;
+		active_size = l;
+	}
+
 	// initialize gradient
 	{
 		G = new double[l];
+		G_bar = new double[l];
 		int i;
 		for(i=0;i<l;i++)
+		{
 			G[i] = b[i];
+			G_bar[i] = 0;
+		}
 		for(i=0;i<l;i++)
 			if(!is_lower_bound(i))
 			{
-				double *Q_i = Q.get_Q(i);
+				double *Q_i = Q.get_Q(i,l);
 				double alpha_i = alpha[i];
-				for(int j=0;j<l;j++)
+				int j;
+				for(j=0;j<l;j++)
 					G[j] += alpha_i*Q_i[j];
+				if(is_upper_bound(i))
+					for(j=0;j<l;j++)
+						G_bar[j] += Q_i[j];
 			}
 	}
 
 	// optimization step
 
 	int iter = 0;
-	int counter = 0;
+	int counter = min(l,1000)+1;
 
 	while(1)
 	{
+		// show progress and do shrinking
+
+		if(--counter == 0)
+		{
+			counter = min(l,1000);
+			if(shrinking) do_shrinking();
+			fprintf(stderr,".");
+		}
+
 		int i,j;
 		if(select_working_set(i,j)!=0)
-			break;
-
-		++iter;
-
-		// show progress
-
-		if(++counter == 100)
 		{
-			fprintf(stderr,".");
-			counter = 0;
+			// reconstruct the whole gradient
+			reconstruct_gradient();
+			// reset active set size and check
+			active_size = l;
+			fprintf(stderr,"*");
+			if(select_working_set(i,j)!=0)
+				break;
+			else
+				counter = 1;	// do shrinking next iteration
 		}
+		
+		++iter;
 
 		// update alpha[i] and alpha[j]
 		
-		const double *Q_i = Q.get_Q(i);
-		const double *Q_j = Q.get_Q(j);
+		const double *Q_i = Q.get_Q(i,active_size);
+		const double *Q_j = Q.get_Q(j,active_size);
 
 		double old_alpha_i = alpha[i];
 		double old_alpha_j = alpha[j];
-		double s = y[i]*old_alpha_i + y[j]*old_alpha_j;
 
-		double H = s/y[j];
-		double L = (s-C*y[i])/y[j];
+		if(y[i]!=y[j])
+		{
+			double L = max(0.0,alpha[j]-alpha[i]);
+			double H = min(C,C+alpha[j]-alpha[i]);
+			alpha[j] += (-G[i]-G[j])/(Q_i[i]+Q_j[j]+2*Q_i[j]);
+			if(alpha[j] >= H) alpha[j] = H;
+			else if(alpha[j] <= L) alpha[j] = L;
+			alpha[i] += (alpha[j] - old_alpha_j);
+		}
+		else
+		{
+			double L = max(0.0,alpha[i]+alpha[j]-C);
+			double H = min(C,alpha[i]+alpha[j]);
+			alpha[j] += (G[i]-G[j])/(Q_i[i]+Q_j[j]-2*Q_i[j]);
+			if(alpha[j] >= H) alpha[j] = H;
+			else if(alpha[j] <= L) alpha[j] = L;			
+			alpha[i] -= (alpha[j] - old_alpha_j);
+		}
 
-		if(y[i]*y[j] < 0) { double t = H; H = L; L = t;}
+		// update alpha_status and G_bar
 
-		H = min(C,H);
-		L = max(0.0,L);
+		{
+			bool ui = is_upper_bound(i);
+			bool uj = is_upper_bound(j);
+			update_alpha_status(i);
+			update_alpha_status(j);
+			int k;
+			if(ui != is_upper_bound(i))
+			{
+				Q_i = Q.get_Q(i,l);
+				if(ui)
+					for(k=0;k<l;k++)
+						G_bar[k] -= Q_i[k];
+				else
+					for(k=0;k<l;k++)
+						G_bar[k] += Q_i[k];
+			}
 
-		alpha[j] += y[i] * (y[j]*G[i] - y[i]*G[j]) /
-			(y[i]*(y[i]*Q_j[j] - 2*Q_i[j]*y[j]) + y[j]*y[j]*Q_i[i]);
-
-		if(alpha[j] > H) alpha[j] = H;
-		else if(alpha[j] < L) alpha[j] = L;
-
-		alpha[i] = (s - y[j]*alpha[j])/y[i];
-
-		// update alpha_status
-
-		update_alpha_status(i);
-		update_alpha_status(j);
+			if(uj != is_upper_bound(j))
+			{
+				Q_j = Q.get_Q(j,l);
+				if(uj)
+					for(k=0;k<l;k++)
+						G_bar[k] -= Q_j[k];
+				else
+					for(k=0;k<l;k++)
+						G_bar[k] += Q_j[k];
+			}
+		}
 
 		// update G
 
 		double delta_alpha_i = alpha[i] - old_alpha_i;
 		double delta_alpha_j = alpha[j] - old_alpha_j;
 		
-		for(int k=0;k<l;k++)
+		for(int k=0;k<active_size;k++)
 		{
 			G[k] += Q_i[k]*delta_alpha_i + Q_j[k]*delta_alpha_j;
 		}
 	}
-	
+
 	// calculate rho
 
-	{
-		double r;
-		int nr_free = 0;
-		double ub = INF, lb = -INF, sum_free = 0;
-		for(int i=0;i<l;i++)
-		{
-			double yG = y[i]*G[i];
-
-			if(is_lower_bound(i))
-			{
-				if(y[i] > 0)
-					ub = min(ub,yG);
-				else
-					lb = max(lb,yG);
-			}
-			else if(is_upper_bound(i))
-			{
-				if(y[i] < 0)
-					ub = min(ub,yG);
-				else
-					lb = max(lb,yG);
-			}
-			else
-			{
-				++nr_free;
-				sum_free += yG;
-			}
-		}
-
-		if(nr_free>0)
-			r = sum_free/nr_free;
-		else
-			r = (ub+lb)/2;
-
-		rho = r;
-	}
+	rho = calculate_rho();
 
 	// calculate objective value
-
 	{
 		double v = 0;
 		int i;
 		for(i=0;i<l;i++)
-			v += alpha[i] * (G[i]/2 + b[i]);
+			v += alpha[i] * (G[i] + b[i]);
 
-		obj = v;
+		obj = v/2;
 	}
 
-	printf("\noptimization finished, #iter = %d\n",iter);
-}
+	// put back the solution
+	{
+		for(int i=0;i<l;i++)
+			alpha_[active_set[i]] = alpha[i];
+	}
 
-Solver::~Solver()
-{
+	// juggle everything back
+	/*{
+		for(int i=0;i<l;i++)
+			while(active_set[i] != i)
+				swap_index(i,active_set[i]);
+				// or Q.swap_index(i,active_set[i]);
+	}*/
+
+	printf("\noptimization finished, #iter = %d\n",iter);
+
+	delete[] b;
+	delete[] y;
+	delete[] alpha;
 	delete[] alpha_status;
+	delete[] active_set;
 	delete[] G;
+	delete[] G_bar;
 }
 
 // return 1 if already optimal, return 0 otherwise
@@ -488,9 +611,9 @@ int Solver::select_working_set(int &out_i, int &out_j)
 	double Gmax2 = -INF;		// max { -grad(f)_i * d | y_i*d = -1 }
 	int Gmax2_idx = -1;
 
-	for(int i=0;i<l;i++)
+	for(int i=0;i<active_size;i++)
 	{
-		if(y[i]>0)	// y > 0
+		if(y[i]==+1)	// y = +1
 		{
 			if(!is_upper_bound(i))	// d = +1
 			{
@@ -509,7 +632,7 @@ int Solver::select_working_set(int &out_i, int &out_j)
 				}
 			}
 		}
-		else		// y < 0
+		else		// y = -1
 		{
 			if(!is_upper_bound(i))	// d = +1
 			{
@@ -538,87 +661,419 @@ int Solver::select_working_set(int &out_i, int &out_j)
 	return 0;
 }
 
+void Solver::do_shrinking()
+{
+	int i,j,k;
+	if(select_working_set(i,j)!=0) return;
+	double Gm1 = -y[j]*G[j];
+	double Gm2 = y[i]*G[i];
+
+	// shrink
+	
+	for(k=0;k<active_size;k++)
+	{
+		if(is_lower_bound(k))
+		{
+			if(y[k]==+1)
+			{
+				if(-G[k] >= Gm1) continue;
+			}
+			else	if(-G[k] >= Gm2) continue;
+		}
+		else if(is_upper_bound(k))
+		{
+			if(y[k]==+1)
+			{
+				if(G[k] >= Gm2) continue;
+			}
+			else	if(G[k] >= Gm1) continue;
+		}
+		else continue;
+
+		--active_size;
+		swap_index(k,active_size);
+		--k;	// look at the newcomer
+	}
+
+	// unshrink, check all variables again before final iterations
+
+	if(unshrinked || -(Gm1 + Gm2) > eps*10) return;
+	
+	unshrinked = true;
+	reconstruct_gradient();
+
+	for(k=l-1;k>=active_size;k--)
+	{
+		if(is_lower_bound(k))
+		{
+			if(y[k]==+1)
+			{
+				if(-G[k] < Gm1) continue;
+			}
+			else	if(-G[k] < Gm2) continue;
+		}
+		else if(is_upper_bound(k))
+		{
+			if(y[k]==+1)
+			{
+				if(G[k] < Gm2) continue;
+			}
+			else	if(G[k] < Gm1) continue;
+		}
+		else continue;
+
+		swap_index(k,active_size);
+		active_size++;
+		++k;	// look at the newcomer
+	}
+}
+
+double Solver::calculate_rho()
+{
+	double r;
+	int nr_free = 0;
+	double ub = INF, lb = -INF, sum_free = 0;
+	for(int i=0;i<active_size;i++)
+	{
+		double yG = y[i]*G[i];
+
+		if(is_lower_bound(i))
+		{
+			if(y[i] > 0)
+				ub = min(ub,yG);
+			else
+				lb = max(lb,yG);
+		}
+		else if(is_upper_bound(i))
+		{
+			if(y[i] < 0)
+				ub = min(ub,yG);
+			else
+				lb = max(lb,yG);
+		}
+		else
+		{
+			++nr_free;
+			sum_free += yG;
+		}
+	}
+
+	if(nr_free>0)
+		r = sum_free/nr_free;
+	else
+		r = (ub+lb)/2;
+
+	return r;
+}
+
+//
+// Solver for nu-svm classification
+//
+
+class Solver_NU_SVC : public Solver
+{
+public:
+	Solver_NU_SVC() {}
+	void Solve(int l, const Kernel& Q, const double *b, const schar *y_,
+		   double *alpha_, double C_, double eps_,
+		   double& obj, double& rho, const schar *y1_, double& r_,
+		   int shrinking)
+	{
+		clone(y1,y1_,l);
+		r = &r_;
+		Solver::Solve(l,Q,b,y_,alpha_,C_,eps_,obj,rho,shrinking);
+	}
+	~Solver_NU_SVC()
+	{
+		delete[] y1;
+	}
+private:
+	int select_working_set(int &i, int &j);
+	double calculate_rho();
+	void do_shrinking();
+	schar *y1;
+	double* r;
+};
+
+int Solver_NU_SVC::select_working_set(int &out_i, int &out_j)
+{
+	// return i,j which maximize -grad(f)^T d , under constraint
+	// if alpha_i == C, d != +1
+	// if alpha_i == 0, d != -1
+
+	double Gmax1 = -INF;	// max { -grad(f)_i * d | y_i = +1, d = +1 }
+	int Gmax1_idx = -1;
+
+	double Gmax2 = -INF;	// max { -grad(f)_i * d | y_i = +1, d = -1 }
+	int Gmax2_idx = -1;
+
+	double Gmax3 = -INF;	// max { -grad(f)_i * d | y_i = -1, d = +1 }
+	int Gmax3_idx = -1;
+
+	double Gmax4 = -INF;	// max { -grad(f)_i * d | y_i = -1, d = -1 }
+	int Gmax4_idx = -1;
+
+	for(int i=0;i<active_size;i++)
+	{
+		if(y1[i]==+1)	// y == +1
+		{
+			if(!is_upper_bound(i))	// d = +1
+			{
+				if(-G[i] > Gmax1)
+				{
+					Gmax1 = -G[i];
+					Gmax1_idx = i;
+				}
+			}
+			if(!is_lower_bound(i))	// d = -1
+			{
+				if(G[i] > Gmax2)
+				{
+					Gmax2 = G[i];
+					Gmax2_idx = i;
+				}
+			}
+		}
+		else		// y == -1
+		{
+			if(!is_upper_bound(i))	// d = +1
+			{
+				if(-G[i] > Gmax3)
+				{
+					Gmax3 = -G[i];
+					Gmax3_idx = i;
+				}
+			}
+			if(!is_lower_bound(i))	// d = -1
+			{
+				if(G[i] > Gmax4)
+				{
+					Gmax4 = G[i];
+					Gmax4_idx = i;
+				}
+			}
+		}
+	}
+
+	if(max(Gmax1+Gmax2,Gmax3+Gmax4) < eps)
+ 		return 1;
+
+	if(Gmax1+Gmax2 > Gmax3+Gmax4)
+	{
+		out_i = Gmax1_idx;
+		out_j = Gmax2_idx;
+	}
+	else
+	{
+		out_i = Gmax3_idx;
+		out_j = Gmax4_idx;
+	}
+	return 0;
+}
+
+void Solver_NU_SVC::do_shrinking()
+{
+	double Gmax1 = -INF;	// max { -grad(f)_i * d | y_i = +1, d = +1 }
+	double Gmax2 = -INF;	// max { -grad(f)_i * d | y_i = +1, d = -1 }
+	double Gmax3 = -INF;	// max { -grad(f)_i * d | y_i = -1, d = +1 }
+	double Gmax4 = -INF;	// max { -grad(f)_i * d | y_i = -1, d = -1 }
+
+	int k;
+	for(k=0;k<active_size;k++)
+	{
+		if(!is_upper_bound(k))
+		{
+			if(y1[k]==+1)
+			{
+				if(-G[k] > Gmax1) Gmax1 = -G[k];
+			}
+			else	if(-G[k] > Gmax3) Gmax3 = -G[k];
+		}
+		if(!is_lower_bound(k))
+		{
+			if(y1[k]==+1)
+			{	
+				if(G[k] > Gmax2) Gmax2 = G[k];
+			}
+			else	if(G[k] > Gmax4) Gmax4 = G[k];
+		}
+	}
+
+	double Gm1 = -Gmax2;
+	double Gm2 = -Gmax1;
+	double Gm3 = -Gmax4;
+	double Gm4 = -Gmax3;
+
+	for(k=0;k<active_size;k++)
+	{
+		if(is_lower_bound(k))
+		{
+			if(y1[k]==+1)
+			{
+				if(-G[k] >= Gm1) continue;
+			}
+			else	if(-G[k] >= Gm3) continue;
+		}
+		else if(is_upper_bound(k))
+		{
+			if(y1[k]==+1)
+			{
+				if(G[k] >= Gm2) continue;
+			}
+			else	if(G[k] >= Gm4) continue;
+		}
+		else continue;
+
+		--active_size;
+		swap(y1[k],y1[active_size]);
+		swap_index(k,active_size);
+		--k;	// look at the newcomer
+	}
+
+	// unshrink, check all variables again before final iterations
+
+	if(unshrinked || max(-(Gm1+Gm2),-(Gm3+Gm4)) > eps*10) return;
+	
+	unshrinked = true;
+	reconstruct_gradient();
+
+	for(k=l-1;k>=active_size;k--)
+	{
+		if(is_lower_bound(k))
+		{
+			if(y[k]==+1)
+			{
+				if(-G[k] < Gm1) continue;
+			}
+			else	if(-G[k] < Gm3) continue;
+		}
+		else if(is_upper_bound(k))
+		{
+			if(y[k]==+1)
+			{
+				if(G[k] < Gm2) continue;
+			}
+			else	if(G[k] < Gm4) continue;
+		}
+		else continue;
+
+		swap(y1[k],y1[active_size]);
+		swap_index(k,active_size);
+		active_size++;
+		++k;	// look at the newcomer
+	}
+}
+
+double Solver_NU_SVC::calculate_rho()
+{
+	int nr_free1 = 0,nr_free2 = 0;
+	double ub1 = INF, ub2 = INF;
+	double lb1 = -INF, lb2 = -INF;
+	double sum_free1 = 0, sum_free2 = 0;
+
+	for(int i=0;i<active_size;i++)
+	{
+		if(y1[i]==+1)
+		{
+			if(is_lower_bound(i))
+				ub1 = min(ub1,G[i]);
+			else if(is_upper_bound(i))
+				lb1 = max(lb1,G[i]);
+			else
+			{
+				++nr_free1;
+				sum_free1 += G[i];
+			}
+		}
+		else
+		{
+			if(is_lower_bound(i))
+				ub2 = min(ub2,G[i]);
+			else if(is_upper_bound(i))
+				lb2 = max(lb2,G[i]);
+			else
+			{
+				++nr_free2;
+				sum_free2 += G[i];
+			}
+		}
+	}
+
+	double r1,r2;
+	if(nr_free1 > 0)
+		r1 = sum_free1/nr_free1;
+	else
+		r1 = (ub1+lb1)/2;
+	
+	if(nr_free2 > 0)
+		r2 = sum_free2/nr_free2;
+	else
+		r2 = (ub2+lb2)/2;
+	
+	*r = (r1+r2)/2;
+	return (r1-r2)/2;
+}
+
 //
 // Q matrices for different formulations
 //
 class C_SVC_Q: public Kernel
 { 
 public:
-	C_SVC_Q(const svm_problem& prob, const svm_parameter& param, const double *_y)
-	:Kernel(prob.l, prob.x, param), l(prob.l), y(_y)
+	C_SVC_Q(const svm_problem& prob, const svm_parameter& param, const schar *y_)
+	:Kernel(prob.l, prob.x, param)
 	{
-		cache = new Cache(l,(int)min((double)l,(param.cache_size*(1<<20))/(sizeof(double)*l)));
+		clone(y,y_,prob.l);
+		cache = new Cache(prob.l,(int)(param.cache_size*(1<<20)));
 	}
 	
-	double *get_Q(int i) const
+	double *get_Q(int i, int len) const
 	{
 		double *data;
-
-		if(cache->get_data(i,&data) == 0)
+		int start;
+		if((start = cache->get_data(i,&data,len)) < len)
 		{
-			for(int j=0;j<l;j++)
+			for(int j=start;j<len;j++)
 				data[j] = y[i]*y[j]*(this->*kernel_function)(i,j);
 		}
 		return data;
 	}
 
+	void swap_index(int i, int j) const
+	{
+		cache->swap_index(i,j);
+		Kernel::swap_index(i,j);
+		swap(y[i],y[j]);
+	}
+
 	~C_SVC_Q()
 	{
+		delete[] y;
 		delete cache;
 	}
 private:
-	const int l;
-	const double *y;
+	schar *y;
 	Cache *cache;
 };
 
-class NU_SVC_Q: public Kernel
-{
-public:
-	NU_SVC_Q(const svm_problem& prob, const svm_parameter& param, const double *_y)
-	:Kernel(prob.l, prob.x, param), l(prob.l), y(_y)
-	{
-		cache = new Cache(l,(int)min((double)l,(param.cache_size*(1<<20))/(sizeof(double)*l)));
-	}
-	
-	double *get_Q(int i) const
-	{
-		double *data;
-
-		if(cache->get_data(i,&data) == 0)
-		{
-			for(int j=0;j<l;j++)
-				data[j] = y[i]*y[j]*(1+(this->*kernel_function)(i,j));
-		}
-		return data;
-	}
-
-	~NU_SVC_Q()
-	{
-		delete cache;
-	}
-private:
-	const int l;
-	const double *y;
-	Cache *cache;
-};
+#define NU_SVC_Q C_SVC_Q	/* same stuff */
 
 class ONE_CLASS_Q: public Kernel
 {
 public:
 	ONE_CLASS_Q(const svm_problem& prob, const svm_parameter& param)
-	:Kernel(prob.l, prob.x, param), l(prob.l)
+	:Kernel(prob.l, prob.x, param)
 	{
-		cache = new Cache(l,(int)min((double)l,(param.cache_size*(1<<20))/(sizeof(double)*l)));
+		cache = new Cache(prob.l,(int)(param.cache_size*(1<<20)));
 	}
 	
-	double *get_Q(int i) const
+	double *get_Q(int i, int len) const
 	{
 		double *data;
-
-		if(cache->get_data(i,&data) == 0)
+		int start;
+		if((start = cache->get_data(i,&data,len)) < len)
 		{
-			for(int j=0;j<l;j++)
+			for(int j=start;j<len;j++)
 				data[j] = (this->*kernel_function)(i,j);
 		}
 		return data;
@@ -629,7 +1084,6 @@ public:
 		delete cache;
 	}
 private:
-	const int l;
 	Cache *cache;
 };
 
@@ -637,29 +1091,41 @@ class C_SVR_Q: public Kernel
 { 
 public:
 	C_SVR_Q(const svm_problem& prob, const svm_parameter& param)
-	:Kernel(prob.l, prob.x, param), l(prob.l)
+	:Kernel(prob.l, prob.x, param)
 	{
-		cache = new Cache(2*l,(int)min((double)(2*l),(param.cache_size*(1<<20))/(sizeof(double)*(2*l))));
+		int l = prob.l;
+		cache = new Cache(2*l,(int)(param.cache_size*(1<<20)));
+		sign = new schar[2*l];
+		index = new int[2*l];
+		for(int k=0;k<l;k++)
+		{
+			sign[k] = 1;
+			sign[k+l] = -1;
+			index[k] = k;
+			index[k+l] = k;
+		}
+	}
+
+	void swap_index(int i, int j) const
+	{
+		cache->swap_index(i,j);
+		swap(sign[i],sign[j]);
+		swap(index[i],index[j]);
 	}
 	
-	double *get_Q(int i) const
+	double *get_Q(int i, int len) const
 	{
 		double *data;
-
-		if(cache->get_data(i,&data) == 0)
+		int start;
+		if((start = cache->get_data(i,&data,len)) < len)
 		{
-			if(i<l)
-				for(int j=0;j<l;j++)
-				{
-					data[j] = (this->*kernel_function)(i,j);
-					data[j+l] = -data[j];
-				}
-			else
-				for(int j=0;j<l;j++)
-				{
-					data[j+l] = (this->*kernel_function)(i-l,j);
-					data[j] = -data[j+l];
-				}
+			schar si = sign[i];
+			for(int j=start;j<len;j++)
+			{
+				double v = (this->*kernel_function)(index[i],index[j]);
+				if(si == sign[j]) data[j] = v;
+				else data[j] = -v;
+			}
 		}
 		return data;
 	}
@@ -667,11 +1133,13 @@ public:
 	~C_SVR_Q()
 	{
 		delete cache;
+		delete[] sign;
+		delete[] index;
 	}
 private:
-	const int l;
-	const double *y;
 	Cache *cache;
+	schar *sign;
+	int *index;
 };
 
 //
@@ -692,20 +1160,24 @@ struct svm_model
 
 static void solve_c_svc(
 	const svm_problem *prob, const svm_parameter* param,
-	double *alpha, double& obj, double& rho)
+	double *alpha, double& obj, double& rho, double& upper_bound)
 {
 	int l = prob->l;
 	double *minus_ones = new double[l];
+	schar *y = new schar[l];
+
 	int i;
 
 	for(i=0;i<l;i++)
 	{
 		alpha[i] = 0;
 		minus_ones[i] = -1;
+		if(prob->y[i] > 0) y[i] = +1; else y[i]=-1;
 	}
 
-	Solver s(l, C_SVC_Q(*prob,*param,prob->y), minus_ones, prob->y,
-		 alpha, param->C, param->eps, obj, rho);
+	Solver s;
+	s.Solve(l, C_SVC_Q(*prob,*param,y), minus_ones, y,
+		alpha, param->C, param->eps, obj, rho, param->shrinking);
 
 	double sum_alpha=0;
 	for(i=0;i<l;i++)
@@ -713,32 +1185,62 @@ static void solve_c_svc(
 
 	printf("nu = %f\n", sum_alpha/(param->C*prob->l));
 
-	delete[] minus_ones;
-
 	for(i=0;i<l;i++)
-		alpha[i] *= prob->y[i];
+		alpha[i] *= y[i];
+
+	upper_bound = param->C;
+
+	delete[] minus_ones;
+	delete[] y;
 }
 
 static void solve_nu_svc(
 	const svm_problem *prob, const svm_parameter *param,
-	double *alpha, double& obj, double& rho)
+	double *alpha, double& obj, double& rho, double& upper_bound)
 {
-	int l = prob->l;
-	double *zeros = new double[l];
-	double *ones = new double[l];
 	int i;
+	int l = prob->l;
+	double nu = param->nu;
 
-	int n = (int)(param->nu*prob->l);	// # of alpha's at upper bound
-	if(n>=prob->l)
+	int y_pos = 0;
+	int y_neg = 0;
+	schar *y = new schar[l];
+
+	for(i=0;i<l;i++)
+		if(prob->y[i]>0)
+		{
+			y[i] = +1;
+			++y_pos;
+		}
+		else
+		{
+			y[i] = -1;
+			++y_neg;
+		}
+
+	if(nu < 0 || nu*l/2 > min(y_pos,y_neg))
 	{
-		fprintf(stderr,"nu must be in (0,1)\n");
+		fprintf(stderr,"specified nu is infeasible\n");
 		exit(1);
 	}
-	for(i=0;i<n;i++)
-		alpha[i] = 1;
-	alpha[n] = param->nu * prob->l - n;
-	for(i=n+1;i<l;i++)
-		alpha[i] = 0;
+
+	double sum_pos = nu*l/2;
+	double sum_neg = nu*l/2;
+
+	for(i=0;i<l;i++)
+		if(y[i] == +1)
+		{
+			alpha[i] = min(1.0,sum_pos);
+			sum_pos -= alpha[i];
+		}
+		else
+		{
+			alpha[i] = min(1.0,sum_neg);
+			sum_neg -= alpha[i];
+		}
+
+	double *zeros = new double[l];
+	schar *ones = new schar[l];
 
 	for(i=0;i<l;i++)
 	{	
@@ -746,25 +1248,32 @@ static void solve_nu_svc(
 		ones[i] = 1;
 	}
 
-	Solver s(l, NU_SVC_Q(*prob,*param,prob->y), zeros, ones,
-		 alpha, 1.0, param->eps, obj, rho);
+	double r;
+	Solver_NU_SVC s;
+	s.Solve(l, NU_SVC_Q(*prob,*param,y), zeros, ones,
+		alpha, 1.0, param->eps, obj, rho, y, r, param->shrinking);
 
-	printf("C = %f\n",1/rho);
-
-	delete[] zeros;
-	delete[] ones;
+	printf("C = %f\n",1/r);
 
 	for(i=0;i<l;i++)
-		alpha[i] *= prob->y[i]/rho;
+		alpha[i] *= y[i]/r;
+
+	rho /= r;
+	obj /= (r*r);
+	upper_bound = 1/r;
+
+	delete[] y;
+	delete[] zeros;
+	delete[] ones;
 }
 
 static void solve_one_class(
 	const svm_problem *prob, const svm_parameter *param,
-	double *alpha, double& obj, double& rho)
+	double *alpha, double& obj, double& rho, double& upper_bound)
 {
 	int l = prob->l;
 	double *zeros = new double[l];
-	double *ones = new double[l];
+	schar *ones = new schar[l];
 	int i;
 
 	int n = (int)(param->nu*prob->l);	// # of alpha's at upper bound
@@ -785,8 +1294,11 @@ static void solve_one_class(
 		ones[i] = 1;
 	}
 
-	Solver s(l, ONE_CLASS_Q(*prob,*param), zeros, ones,
-		 alpha, 1.0, param->eps, obj, rho);
+	Solver s;
+	s.Solve(l, ONE_CLASS_Q(*prob,*param), zeros, ones,
+		alpha, 1.0, param->eps, obj, rho, param->shrinking);
+
+	upper_bound = 1;
 
 	delete[] zeros;
 	delete[] ones;
@@ -794,12 +1306,12 @@ static void solve_one_class(
 
 static void solve_c_svr(
 	const svm_problem *prob, const svm_parameter *param,
-	double *alpha, double& obj, double& rho)
+	double *alpha, double& obj, double& rho, double & upper_bound)
 {
 	int l = prob->l;
 	double *alpha2 = new double[2*l];
 	double *linear_term = new double[2*l];
-	double *y = new double[2*l];
+	schar *y = new schar[2*l];
 	int i;
 
 	for(i=0;i<l;i++)
@@ -813,12 +1325,15 @@ static void solve_c_svr(
 		y[i+l] = -1;
 	}
 
-	Solver s(2*l, C_SVR_Q(*prob,*param), linear_term, y,
-		 alpha2, param->C, param->eps, obj, rho);
+	Solver s;
+	s.Solve(2*l, C_SVR_Q(*prob,*param), linear_term, y,
+		alpha2, param->C, param->eps, obj, rho, param->shrinking);
 
 	for(i=0;i<l;i++)
 		alpha[i] = alpha2[i] - alpha2[i+l];
 
+	upper_bound = param->C;
+
 	delete[] alpha2;
 	delete[] linear_term;
 	delete[] y;
@@ -832,20 +1347,20 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 	svm_model *model = (svm_model *)malloc(sizeof(svm_model));
 	model->param = *param;
 	double *alpha = new double[prob->l];
-	double obj, rho;
+	double obj, rho, upper_bound;
 	switch(param->svm_type)
 	{
 		case C_SVC:
-			solve_c_svc(prob,param,alpha,obj,rho);
+			solve_c_svc(prob,param,alpha,obj,rho,upper_bound);
 			break;
 		case NU_SVC:
-			solve_nu_svc(prob,param,alpha,obj,rho);
+			solve_nu_svc(prob,param,alpha,obj,rho,upper_bound);
 			break;
 		case ONE_CLASS:
-			solve_one_class(prob,param,alpha,obj,rho);
+			solve_one_class(prob,param,alpha,obj,rho,upper_bound);
 			break;
 		case C_SVR:
-			solve_c_svr(prob,param,alpha,obj,rho);
+			solve_c_svr(prob,param,alpha,obj,rho,upper_bound);
 			break;
 	}
 
@@ -861,7 +1376,7 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 		if(fabs(alpha[i]) >= EPS_A)
 		{
 			++nSV;
-			if(fabs(alpha[i]) >= param->C - EPS_A)
+			if(fabs(alpha[i]) >= upper_bound - EPS_A)
 				++nBSV;
 		}
 	}
@@ -890,31 +1405,17 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 	return model;
 }
 
-int svm_classify(const svm_model *model, const svm_node *x,
-		 double label, double *decision_value)
+double svm_classify(const svm_model *model, const svm_node *x)
 {
 	const int n = model->n;
 	const double *sv_coef = model->sv_coef;
 
 	double sum = 0;
-	if(model->param.svm_type == NU_SVC)
-	{
-		for(int i=0;i<n;i++)
-			sum += sv_coef[i] * (1+Kernel::k_function(x,model->SV[i],model->param));
-	}
-	else
-	{
-		for(int i=0;i<n;i++)
-			sum += sv_coef[i] * Kernel::k_function(x,model->SV[i],model->param);
-		sum-=(model->rho);
-	}
 
-	*decision_value = sum;
-
-	if(model->param.svm_type == ONE_CLASS)
-		return (sum > 0);
-	else
-		return (sum*label)>0;
+	for(int i=0;i<n;i++)
+		sum += sv_coef[i] * Kernel::k_function(x,model->SV[i],model->param);
+	sum-=(model->rho);
+	return sum;
 }
 
 const char *svm_type_table[] =
@@ -946,10 +1447,7 @@ int svm_save_model(const char *model_file_name, const svm_model *model)
 	if(param.kernel_type == POLY || param.kernel_type == SIGMOID)
 		fprintf(fp,"coef0 %g\n", param.coef0);
 
-	if(param.svm_type == C_SVC || param.svm_type == ONE_CLASS ||
-	   param.svm_type == C_SVR)
-		fprintf(fp, "rho %g\n", model->rho);
-
+	fprintf(fp, "rho %g\n", model->rho);
 	fprintf(fp, "SV %d\n", model->n);
 
 	const int n = model->n;
